<?php
/**
* @copyright	Copyright (C) 2009 - 2015 Ready Bytes Software Labs Pvt. Ltd. All rights reserved.
* @license		GNU/GPL, see LICENSE.php
* @package		PayPlans
* @subpackage	MES
* @contact 		support+payplans@readybytes.in
*/
if(defined('_JEXEC')===false) die();

class PayplansAppMes extends PayplansAppPayment
{
	protected $_location	= __FILE__;
	
 	public function isApplicable( $refObject= '', $eventName='')
	{
     	if($eventName == 'onPayplansMesStoreCard'){
        	return true;
         }
         return parent::isApplicable($refObject, $eventName);
	}
	
	// Refund Supported by app
 	public function supportForRefund()
    {
    	return true;
    }
    
	// App support payment cancel
	public function isSupportPaymentCancellation($invoice)
	{
		if($invoice->isRecurring()){
			return true;
		}
		return false;
	}
	
	public function onPayplansPaymentForm(PayplansPayment $payment, $data = null)
	{
		if(is_object($data)){
			$data = (array)$data;
		}
		
		$invoice 		=	$payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		$plans			= 	$payment->getPlans(PAYPLANS_INSTANCE_REQUIRE);
		$plan 			=   array_shift($plans);
		$amount 		=   $invoice->getTotal() ;
		$payment_key 	=   $payment->getKey();
	
		$this->assign('post_url',	 	XiRoute::_("index.php?option=com_payplans&view=payment&task=complete&payment_key=".$payment_key));
		$this->assign('cancel_url', 	XiRoute::_("index.php?option=com_payplans&gateway=mes&view=payment&task=complete&action=cancel&payment_key=".$payment_key));
		$this->assign('invoice',   	 	$invoice);
    	$this->assign('payment',   	 	$payment);
		$this->assign('invoice_key', 	$invoice->getKey());
		$this->assign('payment_key', 	$payment_key);
		$this->assign('amount', 		$amount);
				
        return $this->_render('form');
	}
	
	public function onPayplansPaymentAfter(PayplansPayment $payment, &$action, &$data, $controller)
	{
		if($action == 'cancel'){
			return true;
		}
		
		$invoice = $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$card_id	= $this->_storeCardData($data);
		if(!$card_id){
			$getApp 	= XiFactory::getApplication();
			$getApp->enqueueMessage(XiText::_('COM_PAYPLANS_APP_MES_INCORRECT_CARD_DETAILS'),'error');
	  		$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}
		
		$response       = $this->_getPayment($data, $card_id, $invoice);
		
		// If Payment not completed
		if(!$response->isApproved())
		{
			$message	= XiText::_('COM_PAYPLANS_APP_MES_LOGGER_ERROR_IN_PAYMENT');
			$error		= $response->ResponseFields['error_code'].'('.$response->ResponseFields['auth_response_text'].')';
			$response	= $response->ResponseFields;
			$log_id 	= PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $response, 'PayplansPaymentFormatter', '', true);
			$getApp 	= XiFactory::getApplication();
			$getApp->enqueueMessage(XiText::_($error),'error');
	  		$getApp->redirect(XiRoute::_('index.php?option=com_payplans&view=payment&task=pay&payment_key='.$payment->getKey()));
		}
		
		//For Checking Existing Transactions
		$transactions 	= $this->_getExistingTransaction($invoice->getId(), $response->ResponseFields['transaction_id'], $response->ResponseFields['auth_code'], 0);
	 	if(!empty($transactions)){
			return true;
	 	}
	 			
		$recurring = $invoice->isRecurring();
		if($recurring){
			 $recurrence_count = $this->_getRecurrenceCount($invoice, $recurring);
             $this->_processRecurringPayment($payment, $invoice, $recurrence_count, $response, $card_id);
		}else {
			$this->_processNonRecurringPayment($payment, $invoice, $response, $card_id);
		}
			
		return parent::onPayplansPaymentAfter($payment, $action, $data, $controller);	
	}
	
	// define Profile ID, Profile Key and Host Uri for Mes
	public function _defineCredentials()
	{
		require_once 'include.php';
		
		define("HOSTURI", "https://cert.merchante-solutions.com/mes-api/tridentApi");
		define("PROFILEID", $this->getAppParam('profile_id'));
		define("PROFLIEKEY", $this->getAppParam('profile_key'));
	}
	
	// Store Card Data at Mes
	public function _storeCardData($data)
	{
		$this->_defineCredentials();
		
		$response	= new TpgStoreData(PROFILEID, PROFLIEKEY);
		$response->setRequestField('card_number',   $data['card_number']);
		$response->setRequestField('card_exp_date', $data['exp_month'].substr($data['exp_year'], -2));
		$response->setHost(HOSTURI);
		$response->execute();
		
		if($response->isApproved()){
			$card_id = $response->ResponseFields['transaction_id'];
			return $card_id;
		}
		return false;
	}
	
	// Get Payment from MeS
	public function _getPayment($data, $card_id, $invoice)
	{
		$this->_defineCredentials();
			
		$response	= new TpgSale(PROFILEID, PROFLIEKEY);
		$response->setRequestField('card_id', $card_id);
		
		if(!empty($data['cardholder_street_address']) && !empty($data['cardholder_zip'])){
			$response->setAvsRequest($data['cardholder_street_address'], $data['cardholder_zip']);
		}
		
		if(!empty($data['invoice_number'])){
			$response->setRequestField('invoice_number', $data['invoice_number']);
		}
		
		if(!empty($data['cvv2'])){
			$response->setRequestField('cvv2', $data['cvv2']);
		}
		$response->setRequestField('transaction_amount', $data['transaction_amount']);
		
		if($invoice->isRecurring()){
			$response->setEcommInd('3');
		}
		
		$response->setHost( HOSTURI );
		$response->execute();
		
		return $response;
	}
	
 	// Triggered when a subscription is about to expire.
    public function processPayment(PayplansPayment $payment, $invoiceCount)
    {
        //this is the previous payment object?
        $invoice 		= $payment->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
        $invoice_count 	= $invoiceCount + 1;

        if($invoice->isRecurring())
        {
            $recurrence_count 	= $payment->getGatewayParam('pending_recur_count');

            if($recurrence_count > 0)
            {
                $card_id 	= $payment->getGatewayParam('card_id');
                $data		= array();
                $data['transaction_amount']	= number_format($invoice->getTotal($invoice_count), 2);
                
                $result = $this->_getPayment($data, $card_id, $invoice);

                //ewayTrxnError,ewayTrxnStatus,ewayTrxnNumber,ewayReturnAmount,ewayAuthCode
                if(!$result->isApproved())
                {
                	$message	= XiText::_('COM_PAYPLANS_APP_MES_LOGGER_ERROR_IN_PAYMENT');
                    $error		= $result->ResponseFields;
                    PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);

                }else {
                    $this->_processRecurringPayment($payment, $invoice, $recurrence_count, $result, $card_id);
                }
            }
        }
		return true;
    }
    
    // Get Recurrence Count
	private function _getRecurrenceCount(PayplansInvoice $invoice, $recurring)
    {
        $count = $invoice->getRecurrenceCount();
        if(intval($count) === 0){
            return 9999;
        }
        
        // Recurrence Count For Regular Recurring Plan
        if($recurring == PAYPLANS_RECURRING){
        	$recurrence_count = $invoice->getRecurrenceCount();
        }
            
        // Recurrence Count For Recurring + Trial 1 Plan
        if($recurring == PAYPLANS_RECURRING_TRIAL_1){
            $recurrence_count = $invoice->getRecurrenceCount() + 1;
        }
        
        // Recurrence Count For Recurring + Trial 2 Plan
        if($recurring == PAYPLANS_RECURRING_TRIAL_2){
             $recurrence_count = $invoice->getRecurrenceCount() + 2;
        }
        
        return $recurrence_count;
    }
	
	// Recurring Payment
	private function _processRecurringPayment(PayplansPayment $payment, PayplansInvoice $invoice, $recurrence_count, $data, $card_id)
    {
        $recurrence_count 	    = $recurrence_count - 1;
        $amount					= number_format($invoice->getTotal(), 2);

        $transaction            = PayplansTransaction::getInstance();
        $transaction->set('user_id', 			$invoice->getBuyer())
            		->set('invoice_id', 		$invoice->getId())
		            ->set('payment_id', 		$payment->getId())
		            ->set('amount', 			isset($data->RequestFields['transaction_amount']) ? $data->RequestFields['transaction_amount'] : 0)
		            ->set('message',            'COM_PAYPLANS_PAYMENT_MES_RECURRING_PAYMENT_COMPLETED_SUCCESSFULLY')
					->set('gateway_txn_id', 	isset($data->ResponseFields['transaction_id']) 		? $data->ResponseFields['transaction_id'] 		: 0)
					->set('gateway_subscr_id', 	isset($data->ResponseFields['auth_code']) 	   		? $data->ResponseFields['auth_code'] 			: 0)
					->set('gateway_parent_txn', 0)
		            ->set('params', 			PayplansHelperParam::arrayToIni($data->ResponseFields))
            		->save();

        $payment->getGatewayParams()->set('pending_recur_count',$recurrence_count);
		$payment->getGatewayParams()->set('card_id',$card_id);
		$payment->save();
    }

    // Non-Recurring Payment
    private function _processNonRecurringPayment(PayplansPayment $payment, PayplansInvoice $invoice, $data, $card_id)
    {
		$transaction 	= PayplansTransaction::getInstance();
		$transaction->set('user_id', 			$invoice->getBuyer())
					->set('invoice_id', 		$invoice->getId())
					->set('payment_id', 		$payment->getId())
					->set('amount',				number_format($invoice->getTotal(), 2))
					->set('message',            'COM_PAYPLANS_PAYMENT_MES_PAYMENT_COMPLETED_SUCCESSFULLY')
					->set('gateway_txn_id', 	isset($data->ResponseFields['transaction_id']) 		? $data->ResponseFields['transaction_id'] 		: 0)
					->set('gateway_subscr_id', 	isset($data->ResponseFields['auth_code']) 	   		? $data->ResponseFields['auth_code'] 			: 0)
					->set('gateway_parent_txn', 0)
					->set('params', 			PayplansHelperParam::arrayToIni($data->ResponseFields))
					->save();	

    }
    
    // Refund Transaction
 	public function refundRequest(PayplansTransaction $transaction, $refund_amount)
	{
		$invoice = $transaction->getInvoice(PAYPLANS_INSTANCE_REQUIRE);
		
		$this->_defineCredentials();
		
		$tranId 			= $transaction->getGatewayTxnId();
		$refund_amount  	= number_format($refund_amount, 2);
		$subscriptionId 	= $transaction->getGatewaySubscriptionId();
						
		$response		= new TpgRefund(PROFILEID, PROFLIEKEY, $tranId);
		$response->setHost(HOSTURI );
		$response->execute();
		
		//refund response transaction status is true then terminate the order
		if($response->isApproved())
		{	
			$payment 	= $transaction->getPayment();
    	    		
    		if($payment)
    		{
    			 $newtransaction = PayplansTransaction::getInstance();
				 $newtransaction->set('user_id', 			$transaction->getBuyer())
								->set('invoice_id', 		$transaction->getInvoice())
								->set('payment_id',		 	$payment)
								->set('gateway_txn_id',	 	isset($response->ResponseFields['transaction_id']) 		? $response->ResponseFields['transaction_id'] 	: 0)
								->set('gateway_subscr_id', 	$subscriptionId)
								->set('gateway_parent_txn', 0)
								->set('params', 			PayplansHelperParam::arrayToIni($response->ResponseFields));

				$negativeAmt = -($refund_amount);
		    	$newtransaction->set('amount',$negativeAmt)
		    					->set('message', 'COM_PAYPLANS_APP_MES_TRANSACTION_COMPLETED')
		    					->save();
    		}
		    		
    		return true; 
		}else {
			$message	= XiText::_('COM_PAYPLANS_APP_MES_LOGGER_ERROR_IN_PAYMENT');
            $error		= $response->ResponseFields;
            PayplansHelperLogger::log(XiLogger::LEVEL_ERROR, $message, $payment, $error, 'PayplansPaymentFormatter', '', true);
		}
		return false;
	}
	
    
	public function onPayplansPaymentTerminate(PayplansPayment $payment, $controller)
	{
		parent::onPayplansPaymentTerminate($payment, $controller);
		return $this->_render('cancel_success');
	}
	
	// Update Credit card details
	public function onPayplansMesStoreCard($cardNumber, $expMonth, $expYear, $invoiceId)
	{
		$data		= array('card_number' => $cardNumber, 'exp_month' => $expMonth, 'exp_year'=> $expYear);
		
		$card_id 	= $this->_storeCardData($data);
		if($card_id){
			
			$invoice  	 = PayplansInvoice::getInstance($invoiceId);
	        $payment   	 = $invoice->getPayment();
	        
	       	$payment->getGatewayParams()->set('card_id',$card_id);
			$payment->save();
			
			return true;
		}
		return false;
	}
	
}



